@@ -2,4 +2,4 @@ globalVariables(c("value", "sigma", "condition", "x", "y", "name", "proflist", "
                   "sigmaLS", "cbLower95", "cbUpper95", "cbLower68", "cbUpper68",
                   "iteration", "idx", "is.zero", "index", "converged", "iterations",
                   ".runbgOutput", "terminal", "parvalue", "dataErrorModel",
-                  "Rate"))
+                  "Rate", "weighted.residual"))
