@@ -34,6 +34,3 @@ plotValues(parframe)
 ## Reduce parframe to best fit
 bestfit <- as.parvec(parframe)
 plot((g*x)(times, bestfit), data)
-
-## Plot array with all solutions
-plotArray(parframe, g*x, times, data)
